import { encodeFunctionData, keccak256, toBytes, zeroAddress, } from 'viem';
import AccountFactoryAbi from '../abis/AccountFactory';
import { SMART_ACCOUNT_FACTORY_ADDRESS, VALIDATOR_ADDRESS } from '../constants';
import { getInitializerCalldata, getSmartAccountAddressFromInitialSigner, isSmartAccountDeployed, } from '../utils';
export async function deployAccount(params) {
    const { initialSignerAddress, walletClient, publicClient, paymaster, paymasterInput, } = params;
    const initialSigner = initialSignerAddress ?? walletClient.account.address;
    const address = await getSmartAccountAddressFromInitialSigner(initialSigner, publicClient);
    let deploymentTransaction = undefined;
    const isDeployed = await isSmartAccountDeployed(publicClient, address);
    if (!isDeployed) {
        const initializerCallData = getInitializerCalldata(initialSigner, VALIDATOR_ADDRESS, {
            allowFailure: false,
            callData: '0x',
            value: 0n,
            target: zeroAddress,
        });
        const addressBytes = toBytes(initialSigner);
        const salt = keccak256(addressBytes);
        const deploymentCalldata = encodeFunctionData({
            abi: AccountFactoryAbi,
            functionName: 'deployAccount',
            args: [salt, initializerCallData],
        });
        deploymentTransaction = await walletClient.sendTransaction({
            to: SMART_ACCOUNT_FACTORY_ADDRESS,
            data: deploymentCalldata,
            paymaster,
            paymasterInput,
        });
    }
    return {
        smartAccountAddress: address,
        deploymentTransaction,
    };
}
//# sourceMappingURL=deployAccount.js.map